<?php
/**
 * Plugin Name: Export 2 Multisite
 * Plugin URI: http://wordpress.org/plugins/export-2-multisite/
 * Description: Exports your database as a MySQL data dump that can be imported into a WP Multisite installation. 
 * Author: Hippocurious Interactive Inc
 * Version: 0.2
 * Author URI: http://hippocurious.com
 *
 * Copyright (C) 2014 Hippocurious Interactive Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * NOTICE:
 * This plugin was derived from WP Migrate DB developed by Delicious Brains
 * It has been stripped down and modified to provide the specific functionality 
 * of creating an SQL dump that can be imported to WP Multisite.
**/

$GLOBALS['e2wpms_meta']['e2wpms']['version'] = '0.2';

if ( ! class_exists( 'E2WPMS_Utils' ) ) {
	require dirname( __FILE__ ) . '/class/e2wpms-utils.php';
}

// Define the directory seperator if it isn't already
if( !defined( 'DS' ) ) {
	if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') {
		define('DS', '\\');
	}
	else {
		define('DS', '/');
	}
}

function e2wpms_loaded() {
	// exit quickly unless: standalone admin; multisite network admin; one of our AJAX calls
	if ( ! is_admin() || ( is_multisite() && ! is_network_admin() && ! E2WPMS_Utils::is_ajax() ) ) {
		return;
	}
	e2wpms();
}
add_action( 'plugins_loaded', 'e2wpms_loaded' );

function e2wpms_init() {
	// if neither WordPress admin nor running from wp-cli, exit quickly to prevent performance impact
	if ( !is_admin() && ! ( defined( 'WP_CLI' ) && WP_CLI ) ) return;
}
add_action( 'init', 'e2wpms_init' );

/**
 * Populate the $e2wpms global with an instance of the E2WPMS class and return it.
 *
 * @return E2WPMS The one true global instance of the E2WPMS class.
 */
function e2wpms() {
	global $e2wpms;

	if ( ! is_null( $e2wpms ) ) {
		return $e2wpms;
	}

	$abspath = dirname( __FILE__ );

	require_once $abspath . '/class/e2wpms-base.php';
	require_once $abspath . '/class/e2wpms.php';

	$e2wpms = new E2WPMS( __FILE__ );

	return $e2wpms;
}
